#!/usr/bin/env python

import logging

from lxml import etree
from ncclient import manager
from ncclient.xml_ import *


def connect(host, port, user, password):
    conn = manager.connect(host=host,
                           port=port,
                           username=user,
                           password=password,
                           timeout=10,
                           device_params={'name': 'alu'},
                           hostkey_verify=False)


    logging.info('Retrieving full config, please wait ...')

    result = conn.get_configuration()
    logging.info(result)

    logging.info('Here is the chassis configuration')
    output = result.xpath('data/configure/system/chassis')[0]
    logging.info(to_xml(output))

    logging.info('Retrieving service config')

    # specify filter to pass to get_config
    filter = new_ele('configure', attrs={'xmlns': ALU_CONFIG})
    sub_ele(filter, 'service')
    result = conn.get_configuration(filter=filter)
    epipes = result.xpath('data/configure/service/epipe')

    for i in epipes:
        logging.info(etree.tostring(i, pretty_print=True).decode('utf-8'))

    logging.info('Getting CLI -config')
    cli_cfg = conn.get_configuration(content='cli', filter=['port 1/1/11'])
    logging.info(cli_cfg)

    logging.info('Get detailed CLI -config')
    cli_cfg = conn.get_configuration(content='cli', filter=['port 1/1/11'], detail=True)
    logging.info(cli_cfg)

    conn.close_session()


if __name__ == '__main__':
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s:%(lineno)d %(message)s'
    logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=LOG_FORMAT)

    connect('localhost', 830, 'admin', 'admin')



# Copyright 2h009 Shikhar Bhushan
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"Locking-related NETCONF operations"

from ncclient.xml_ import *

from ncclient.operations.rpc import RaiseMode, RPC

# TODO: parse session-id from a lock-denied error, and raise a tailored exception?

class Lock(RPC):

    "`lock` RPC"


    def request(self, target="candidate"):
        """Allows the client to lock the configuration system of a device.

        *target* is the name of the configuration datastore to lock
        """
        node = new_ele("lock")
        sub_ele(sub_ele(node, "target"), target)
        return self._request(node)


class Unlock(RPC):

    "`unlock` RPC"

    def request(self, target="candidate"):
        """Release a configuration lock, previously obtained with the lock operation.

        *target* is the name of the configuration datastore to unlock
        """
        node = new_ele("unlock")
        sub_ele(sub_ele(node, "target"), target)
        return self._request(node)


class LockContext:

    """A context manager for the :class:`Lock` / :class:`Unlock` pair of RPC's.

    Any `rpc-error` will be raised as an exception.

    Initialise with (:class:`Session <ncclient.transport.Session>`) instance and lock target.
    """

    def __init__(self, session, device_handler, target):
        self.session = session
        self.target = target
        self.device_handler = device_handler

    def __enter__(self):
        Lock(self.session, self.device_handler, raise_mode=RaiseMode.ERRORS).request(self.target)
        return self

    def __exit__(self, *args):
        Unlock(self.session, self.device_handler, raise_mode=RaiseMode.ERRORS).request(self.target)
        return False



#! /usr/bin/env python
#
# Connect to the NETCONF server passed on the command line and
# set a device_params to turn on/off the namespace prefix "nc".
# if you want to verify the result, you can print the request that
# was sent. For brevity and clarity of the examples, we omit proper
# exception handling.
#
# $ ./ericsson_nc_prefix_example.py host username password

import sys, os, warnings
warnings.simplefilter("ignore", DeprecationWarning)
from ncclient import manager


def ericsson_connect(host, port, user, password, device_params):
    return manager.connect(host=host,
                           port=port,
                           username=user,
                           password=password,
                           device_params=device_params,
                           hostkey_verify-false)


def enable_nc_prefix(host, user, password):
    # add a parameter 'with_ns' to turn on/off 'nc'
    device_params = {'name': 'ericsson', 'with_ns': True}
    with ericsson_connect(host,
                          port=22,
                          user=user,
                          password=password,
                          device_params=device_params) as m:

        ret = m.get_config(source="running").data_xml
        print(ret)


def disable_nc_prefix(host, user, password):
    # add a parameter 'with_ns' to turn on/off 'nc'
    device_params = {'name': 'ericsson', 'with_ns': False}
    with ericsson_connect(host,
                          port=22,
                          user=user,
                          password=password,
                          device_params=device_params) as m:

        ret = m.get_config(source="running").data_xml
        print(ret)


def demo(host, user, password):
    enable_nc_prefix(host, user, password)
    print("#"*50)
    disable_nc_prefix(host, user, password)


if __name__ == '__main__':
    demo(sys.argv[1], sys.argv[2], sys.argv[3])




# Copyright 2009 Shikhar Bhushan
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ncclient.operations.rpc import RPC

from ncclient.xml_ import *

from ncclient.operations import util

class CreateSubscription(RPC):
    "`create-subscription` RPC. Depends on the `:notification` capability."

    DEPENDS = [':notification']

    def request(self, filter=None, stream_name=None, start_time=None, stop_time=None):
        """Creates a subscription for notifications from the server.

        *filter* specifies the subset of notifications to receive (by
        default all notificaitons are received)

        :seealso: :ref:`filter_params`

        *stream_name* specifies the notification stream name. The
        default is None meaning all streams.

        *start_time* triggers the notification replay feature to
        replay notifications from the given time. The default is None,
        meaning that this is not a replay subscription. The format is
        an RFC 3339/ISO 8601 date and time.

        *stop_time* indicates the end of the notifications of
        interest. This parameter must be used with *start_time*. The
        default is None, meaning that (if *start_time* is present) the
        notifications will continue until the subscription is
        terminated. The format is an RFC 3339/ISO 8601 date and time.

        """
        node = new_ele_ns("create-subscription", NETCONF_NOTIFICATION_NS)
        if filter is not None:
            node.append(util.build_filter(filter))
        if stream_name is not None:
            sub_ele_ns(node, "stream", NETCONF_NOTIFICATION_NS).text = stream_name

        if start_time is not None:
            sub_ele_ns(node, "startTime", NETCONF_NOTIFICATION_NS).text = start_time

        if stop_time is not None:
            if start_time is None:
                raise ValueError("You must provide start_time if you provide stop_time")
            sub_ele_ns(node, "stopTime", NETCONF_NOTIFICATION_NS).text = stop_time

        return self._request(node)



# Copyright 2009 Shikhar Bhushan
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"Session-related NETCONF operations"

from ncclient.xml_ import *

from ncclient.operations.rpc import RPC

class CloseSession(RPC):

    "`close-session` RPC. The connection to NETCONF server is also closed."

    def request(self):
        "Request graceful termination of the NETCONF session, and also close the transport."
        ret = self._request(new_ele("close-session"))
        self.session.close()
        return ret


class KillSession(RPC):

    "`kill-session` RPC."

    def request(self, session_id):
        """Force the termination of a NETCONF session (not the current one!)

        *session_id* is the session identifier of the NETCONF session to be terminated as a string
        """
        node = new_ele("kill-session")
        sub_ele(node, "session-id").text = session_id
        return self._request(node)



"""
Handler for Cisco IOS-XE device specific information.

Note that for proper import, the classname has to be:

    "<Devicename>DeviceHandler"

...where <Devicename> is something like "Default", "Nexus", etc.

All device-specific handlers derive from the DefaultDeviceHandler, which implements the
generic information needed for interaction with a Netconf server.

"""

from .default import DefaultDeviceHandler

from ncclient.operations.third_party.iosxe.rpc import SaveConfig
from ncclient.xml_ import BASE_NS_1_0

import logging
logger = logging.getLogger("ncclient.devices.iosxe")


def iosxe_unknown_host_cb(host, fingerprint):
        # This will ignore the unknown host check when connecting to CSR devices
        return True

class IosxeDeviceHandler(DefaultDeviceHandler):
    """
    Cisco IOS-XE handler for device specific information.

    """
    def __init__(self, device_params, ignore_errors=None):
        super(IosxeDeviceHandler, self).__init__(device_params, ignore_errors)

    def add_additional_operations(self):
        dict = {}
        dict["save_config"] = SaveConfig
        return dict
        
    def add_additional_ssh_connect_params(self, kwargs):
        kwargs['unknown_host_cb'] = iosxe_unknown_host_cb

    def transform_edit_config(self, node):
        # find the first node that has the tag "config" with no namespace
        nodes = node.findall("./config")
        if len(nodes) == 1:
            logger.debug('IOS XE handler: patching namespace of config element')
            nodes[0].tag = '{%s}%s' % (BASE_NS_1_0, 'config')
        return node



import logging
import sys
import threading
import time
from ncclient import manager

# Device connection details
DEVICE_HOST = 'localhost'
DEVICE_PORT = 830
DEVICE_USER = 'admin'
DEVICE_PASS = 'admin'

# Configuration payload template for SNMP community string
EDIT_CONFIG_PAYLOAD = """
<config xmlns="urn:ietf:params:xml:ns:netconf:base:1.0">
    <configure xmlns="urn:nokia.com:sros:ns:yang:sr:conf">
        <system>
            <security>
                <snmp>
                    <community>
                        <community-string>TestCommunity-{}</community-string>
                        <access-permissions>r</access-permissions>
                        <version>v2c</version>
                    </community>
                </snmp>
            </security>
        </system>
    </configure>
</config>
"""


def create_ncclient_session(host, port, username, password, config_mode=None):
    """
    Creates and returns an ncclient manager session for connecting to the Nokia SR OS device.

    :param host: Hostname or IP of the Nokia device
    :param port: NETCONF port
    :param username: Username for device authentication
    :param password: Password for device authentication
    :param config_mode: 'private' to use private candidate mode, None for default mode
    :return: NETCONF manager session
    """
    device_params = {'name': 'sros'}
    if config_mode:
        device_params['config_mode'] = config_mode

    return manager.connect(
        host=host,
        port=port,
        username=username,
        password=password,
        hostkey_verify=False,
        device_params=device_params
    )


def configure_and_commit(session_id, config_mode=None):
    """
    Creates a NETCONF session, edits the configuration, and commits the changes.

    :param session_id: Unique session identifier for logging
    :param config_mode: 'private' for private candidate mode, None for default mode
    """
    try:
        with create_ncclient_session(DEVICE_HOST, DEVICE_PORT, DEVICE_USER, DEVICE_PASS, config_mode=config_mode) as nc_session:
            logging.info(f"Session {session_id} started with config_mode: {config_mode or 'default'}")

            # Edit the device configuration using the payload
            edit_payload = EDIT_CONFIG_PAYLOAD.format(session_id)
            nc_session.edit_config(target='candidate', config=edit_payload)
            logging.info(f"Session {session_id} successfully edited the configuration.")

            # Commit the changes to the device
            nc_session.commit(comment=f"Session {session_id} commit in {config_mode or 'default'} mode")
            logging.info(f"Session {session_id} committed successfully in {config_mode or 'default'} mode.")

    except Exception as e:
        logging.error(f"Session {session_id} encountered an error: {e}")


def run_concurrent_sessions(session_ids, config_mode=None):
    """
    Runs multiple NETCONF sessions concurrently.

    :param session_ids: List of session IDs to identify different threads
    :param config_mode: 'private' to use private candidate mode, None for default mode
    """
    threads = []
    for session_id in session_ids:
        thread = threading.Thread(target=configure_and_commit, args=(session_id, config_mode))
        threads.append(thread)
        thread.start()

    # Ensure all threads finish execution
    for thread in threads:
        thread.join()


def main():
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s:%(lineno)d %(message)s'
    logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=LOG_FORMAT)

    logging.info("Starting test with concurrent sessions in default mode (expecting conflicts)...")

    # Run test with concurrent sessions in default mode
    run_concurrent_sessions([1, 2, 3, 4, 5])

    logging.info("Now testing with concurrent sessions in private mode (no conflicts expected)...")

    # Run test with concurrent sessions in private candidate mode
    run_concurrent_sessions([6, 7, 8, 9], config_mode='private')

    logging.info("Test completed.")


if __name__ == '__main__':
    main()



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Hareesh Puthalath, Cisco Systems, Inc.

import sys
import logging
from ncclient import manager

log = logging.getLogger(__name__)

# Various IOS Snippets

CREATE_VRF = """
<config>
        <cli-config-data>
            <cmd>ip routing</cmd>
            <cmd>ip vrf %s</cmd>
        </cli-config-data>
</config>
"""


REMOVE_VRF = """
<config>
        <cli-config-data>
            <cmd>ip routing</cmd>
            <cmd>no ip vrf %s</cmd>
        </cli-config-data>
</config>
"""

CREATE_SUBINTERFACE = """
<config>
        <cli-config-data>
            <cmd>interface %s</cmd>
            <cmd>encapsulation dot1Q %s</cmd>
            <cmd>ip vrf forwarding %s</cmd>
            <cmd>ip address %s %s</cmd>
        </cli-config-data>
</config>

"""

REMOVE_SUBINTERFACE = """
<config>
        <cli-config-data>
            <cmd>no interface %s</cmd>
        </cli-config-data>
</config>
"""


def csr_connect(host, port, user, password):
    return manager.connect(host=host,
                           port=port,
                           username=user,
                           password=password,
                           device_params={'name': "csr"},
                           timeout=30
            )


def create_vrf(conn, vrf_name):
    try:
        confstr = CREATE_VRF % vrf_name
        rpc_obj = conn.edit_config(target='running', config=confstr)
        _check_response(rpc_obj, "CREATE_VRF")
    except Exception:
        log.exception("Exception in creating VRF %s" % vrf_name)


def create_subinterface(conn, subinterface, vlan_id, vrf_name, ip, mask):
    try:
        confstr = CREATE_SUBINTERFACE % (subinterface, vlan_id,
                                         vrf_name, ip, mask)
        rpc_obj = conn.edit_config(target='running', config=confstr)
        _check_response(rpc_obj, 'CREATE_SUBINTERFACE')
    except Exception:
        log.exception("Exception in creating subinterface %s" % subinterface)


def remove_vrf(conn, vrf_name):
    try:
        confstr = REMOVE_VRF % vrf_name
        rpc_obj = conn.edit_config(target='running', config=confstr)
        _check_response(rpc_obj, "REMOVE_VRF")
    except Exception:
        log.exception("Exception in removing VRF %s" % vrf_name)


def remove_subinterface(conn, subinterface):
    try:
        confstr = REMOVE_SUBINTERFACE % (subinterface)
        rpc_obj = conn.edit_config(target='running', config=confstr)
        _check_response(rpc_obj, 'REMOVE_SUBINTERFACE')
    except Exception:
        log.exception("Exception in removing subinterface %s" % subinterface)


def _check_response(rpc_obj, snippet_name):
    log.debug("RPCReply for %s is %s" % (snippet_name, rpc_obj.xml))
    xml_str = rpc_obj.xml
    if "<ok />" in xml_str:
        log.info("%s successful" % snippet_name)
    else:
        log.error("Cannot successfully execute: %s" % snippet_name)


def test_csr(host, user, password):
    with csr_connect(host, port=22, user=user, password=password) as m:
        create_vrf(m, "test_vrf")
        create_subinterface(m, "GigabitEthernet 1.500",
                            '500', 'test_vrf',
                            '192.168.0.1', '255.255.255.0')
        #Optional
        remove_vrf(m, "test_vrf")
        remove_subinterface(m, "GigabitEthernet 1.500")


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test_csr(sys.argv[1], sys.argv[2], sys.argv[3])


# Copyright 2h009 Shikhar Bhushan
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'Power-control operations'

from ncclient.xml_ import *

from ncclient.operations.rpc import RPC

PC_URN = "urn:liberouter:params:xml:ns:netconf:power-control:1.0"

class PoweroffMachine(RPC):

    "*poweroff-machine* RPC (flowmon)"

    DEPENDS = ["urn:liberouter:param:netconf:capability:power-control:1.0"]
    
    def request(self):
        return self._request(new_ele(qualify("poweroff-machine", PC_URN)))

class RebootMachine(RPC):

    "*reboot-machine* RPC (flowmon)"

    DEPENDS = ["urn:liberouter:params:netconf:capability:power-control:1.0"]

    def request(self):
        return self._request(new_ele(qualify("reboot-machine", PC_URN)))



#!/usr/bin/env python
import logging

from ncclient import manager
from ncclient.xml_ import *


def connect(host, port, user, password, source):
    conn = manager.connect(host=host,
                           port=port,
                           username=user,
                           password=password,
                           timeout=60,
                           device_params={'name': 'junos'},
                           hostkey_verify=False)

    logging.info('Retrieving full config, please wait ...')
    result = conn.get_config(source)

    logging.info('Showing \'system\' hierarchy ...')
    output = result.xpath('data/configuration/system')[0]
    logging.info(to_xml(output))

    # specify filter to pass to get_config
    root_filter = new_ele('filter')
    config_filter = sub_ele(root_filter, 'configuration')
    system_filter = sub_ele(config_filter, 'system')
    sub_ele(system_filter, 'services')
    
    filtered_result = conn.get_config(source, filter=root_filter)
    logging.info('Configured Services...')
    for i in filtered_result.xpath('data/configuration/system/services/*'):
        logging.info(' %s', i.tag)

    logging.info('Configured Interfaces...')
    logging.info('%-15s %-30s' % ('Name', 'Description'))
    logging.info('-' * 40)
    interfaces = result.xpath('data/configuration/interfaces/interface')
    for i in interfaces:
        if i.tag == 'interface':
            interface = i.xpath('name')[0].text
            try:
                description = i.xpath('description')[0].text
            except IndexError:
                description = None
            logging.info('%-15s %-30s' % (interface, description))
            units = i.xpath('unit')
            for u in units:
                unit = u.xpath('name')[0].text
                try:
                    u_desc = u.xpath('description')[0].text 
                except IndexError:
                    u_desc = None
                logging.info('   %-12s %-30s' % (unit, u_desc))


if __name__ == '__main__':
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s:%(lineno)d %(message)s'
    logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=LOG_FORMAT)

    connect('router', 830, 'netconf', 'juniper!', 'candidate')



#!/usr/bin/env python

import logging

from ncclient import manager
from ncclient.xml_ import *


def connect(host, port, user, password):
    conn = manager.connect(host=host,
                           port=port,
                           username=user,
                           password=password,
                           timeout=60,
                           device_params={'name': 'junos'},
                           hostkey_verify=False)

    logging.info('locking configuration')
    lock_result = conn.lock()
    logging.info(lock_result)

    peers = {
            '10.1.1.1': '65001',
            '10.2.1.1': '65002',
            '10.3.1.1': '65003',
            '10.4.1.1': '65004',
            '10.5.1.1': '65005'
            }

    # build configuration element
    config = new_ele('protocols')
    config_bgp = sub_ele(config, 'bgp')
    config_group = sub_ele(config_bgp, 'group')
    # TODO: unused variable! Is this example broken?
    config_group_name = sub_ele(config_group, 'name').text = 'NETCONF_GROUP'
    sub_ele(config_group, 'multipath')
    sub_ele(config_group, 'local-address').text = '10.0.0.1'
    for peer in peers:
        config_neighbor = sub_ele(config_group, 'neighbor')
        sub_ele(config_neighbor, 'name').text = peer
        sub_ele(config_neighbor, 'peer-as').text = peers[peer]

    load_config_result = conn.load_configuration(config=config)
    logging.info(load_config_result)

    validate_result = conn.validate()
    logging.info(validate_result)

    compare_config_result = conn.compare_configuration()
    logging.info(compare_config_result)

    conn.commit()
    logging.info('committed configuration')

    discard_changes_result = conn.discard_changes()
    logging.info(discard_changes_result)

    logging.info('unlocking configuration')
    unlock_result = conn.unlock()
    logging.info(unlock_result)


if __name__ == '__main__':
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s:%(lineno)d %(message)s'
    logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=LOG_FORMAT)

    connect('router', 830, 'netconf', 'juniper!')



from ncclient.operations import OperationError
from ncclient.xml_ import *

from ncclient.operations.rpc import RPC

def global_operations(node):
    """Instantiate an SR OS global operation action element

    Args:
        node: A string representing the top-level action for a
            given global operation.
    Returns:
        A tuple of 'lxml.etree._Element' values.  The first value
        represents the top-level YANG action element and the second
        represents the caller supplied initial node.
    """
    parent, child = yang_action('global-operations',
            attrs={'xmlns': SROS_GLOBAL_OPS_NS})
    ele = sub_ele(child, node)
    return (parent, ele)

class MdCliRawCommand(RPC):
    def request(self, command=None):
        node, raw_cmd_node = global_operations('md-cli-raw-command')
        sub_ele(raw_cmd_node, 'md-cli-input-line').text = command
        self._huge_tree = True
        return self._request(node)


class Commit(RPC):
    "`commit` RPC. Depends on the `:candidate` capability, and the `:confirmed-commit`."

    DEPENDS = [':candidate']

    def request(self, confirmed=False, timeout=None, persist=None, persist_id=None, comment=None):
        """Commit the candidate configuration as the device's new current configuration. Depends on the `:candidate` capability.

        A confirmed commit (i.e. if *confirmed* is `True`) is reverted if there is no followup commit within the *timeout* interval. If no timeout is specified the confirm timeout defaults to 600 seconds (10 minutes). A confirming commit may have the *confirmed* parameter but this is not required. Depends on the `:confirmed-commit` capability.

        *confirmed* whether this is a confirmed commit

        *timeout* specifies the confirm timeout in seconds

        *persist* make the confirmed commit survive a session termination, and set a token on the ongoing confirmed commit

        *persist_id* value must be equal to the value given in the <persist> parameter to the original <commit> operation.

        *comment* a descriptive text comment that will be associated with the commit action. This is useful for logging or audit purposes.
        """

        node = new_ele("commit")
        if comment and comment.strip():
            sub_ele(node, "comment",
                    attrs={'xmlns': "urn:nokia.com:sros:ns:yang:sr:ietf-netconf-augments"}).text = comment

        if persist and persist_id:
            raise OperationError("Invalid operation as persist cannot be present with persist-id")
        if confirmed:
            self._assert(":confirmed-commit")
            sub_ele(node, "confirmed")
            if timeout is not None:
                sub_ele(node, "confirm-timeout").text = timeout
            if persist is not None:
                sub_ele(node, "persist").text = persist
        if persist_id:
            sub_ele(node, "persist-id").text = persist_id

        return self._request(node)



#!/usr/bin/env python
import logging
import sys

from ncclient import manager


def connect(host, port, user, password, command):
    with manager.connect(
        host=host,
        port=port,
        username=user,
        password=password,
        timeout=60,
        device_params={'name': 'junos'},
        hostkey_verify=False
    ) as m:
        with m.locked('candidate'):
            result = m.load_configuration(action='set', config=command)
            logging.info(result)
            result = m.commit()
            logging.info(result)


if __name__ == '__main__':
    LOG_FORMAT = '%(asctime)s %(levelname)s %(filename)s:%(lineno)d %(message)s'
    logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=LOG_FORMAT)

    interface = 'em0'
    connect('router', 830, 'netconf', 'juniper!', 'set interfaces %s description example' % interface)



# Copyright 2009 Shikhar Bhushan
# Copyright 2014 Leonidas Poulopoulos
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ncclient import NCClientError

class TransportError(NCClientError):
    pass

class SessionError(NCClientError):
    pass

class AuthenticationError(TransportError):
    pass

class PermissionError(TransportError):
    pass

class SessionCloseError(TransportError):

    def __init__(self, in_buf, out_buf=None):
        msg = 'Unexpected session close'
        if in_buf:
            msg += '\nIN_BUFFER: `%s`' % in_buf
        if out_buf:
            msg += ' OUT_BUFFER: `%s`' % out_buf
        SSHError.__init__(self, msg)

class SSHError(TransportError):
    pass

class SSHUnknownHostError(SSHError):

    def __init__(self, host, fingerprint):
        SSHError.__init__(self, 'Unknown host key [%s] for [%s]' % (fingerprint, host))
        self.host = host
        self.fingerprint = fingerprint

class NetconfFramingError(TransportError):
    pass


class TLSError(TransportError):
    pass

class UnixSocketError (TransportError):
    pass



from .default import DefaultDeviceHandler
from ncclient.operations.third_party.sros.rpc import MdCliRawCommand, Commit
from ncclient.xml_ import BASE_NS_1_0


class ConfigMode:
    PRIVATE = 'private'

def passthrough(xml):
    return xml

class SrosDeviceHandler(DefaultDeviceHandler):
    """
    Nokia SR OS handler for device specific information.
    """

    def __init__(self, device_params, ignore_errors=None):
        super(SrosDeviceHandler, self).__init__(device_params, ignore_errors)

    def get_capabilities(self):
        """Set SR OS device handler client capabilities

        Set additional capabilities beyond the default device handler.

        Returns:
            A list of strings representing NETCONF capabilities to be
            sent to the server.
        """
        base = super(SrosDeviceHandler, self).get_capabilities()
        additional = [
            'urn:ietf:params:xml:ns:netconf:base:1.0',
            'urn:ietf:params:xml:ns:yang:1',
            'urn:ietf:params:netconf:capability:confirmed-commit:1.1',
            'urn:ietf:params:netconf:capability:validate:1.1',
        ]
        if self.device_params.get('config_mode') == ConfigMode.PRIVATE:
            additional.append('urn:nokia.com:nc:pc')

        return base + additional

    def get_xml_base_namespace_dict(self):
        return {None: BASE_NS_1_0}

    def get_xml_extra_prefix_kwargs(self):
        d = {}
        d.update(self.get_xml_base_namespace_dict())
        return {"nsmap": d}

    def add_additional_operations(self):
        operations = {
            'md_cli_raw_command': MdCliRawCommand,
            'commit': Commit,
        }
        return operations

    def perform_qualify_check(self):
        return False

    def transform_reply(self):
        return passthrough
